
#######################################################################################
### R code
### Helbich M 2016 Data on Healthy Food Accessibility in Amsterdam, The Netherlands. Data. 
### Data used in Helbich M et al. 2017 (Food Deserts? Healthy Food Access in Amsterdam)
#######################################################################################

# load packages
library(sp); library(rgdal)
library(spdep); require(kohonen)
library(RColorBrewer)

# set workspace
setwd("C:/workspace")
getwd()
# Download zip from the web
#download.file("http://ADD_LINK/data_amsterdam.zip", "data_amsterdam.zip")
# unzip file
unzip("data_amsterdam.zip")

# load shapefile
data <- readOGR(".", "data_amsterdam")
# save(data,file="data_amsterdam")
# projection of the data
proj4string(data)
# bounding box
bbox(data)

# show attributes
names(data)
# summary attributes
summary(data[, c("NATI", "HOUS", "PROX", "DENS", "VARI")])
str(data@data)

# select variables for som-based clustering
data.som <- as.matrix(data@data[, c("NATI", "HOUS", "PROX", "DENS", "VARI")])
# scale selected variables 
data.som.sc <- scale(data.som)

# create som topology
som.grid <- somgrid(xdim = 10, ydim = 8, topo = c("hexagonal"))
set.seed(20082014)
# train som
som.res <- som(data=data.som.sc, grid=som.grid, rlen=100, alpha = c(0.05, 0.01), 
               keep.data=TRUE, n.hood="circular")
summary(som.res)

# set colors
coolBlueHotRed <- function(n, alpha = 1) { rainbow(n, end=4/6, alpha=alpha)[n:1] }
# plot som training progress
plot(som.res, type="changes")
# plot how many cells are mapped to each neuron
plot(som.res, type="counts", palette.name=coolBlueHotRed)
# U-Matrix to explore clusters
plot(som.res, type="dist.neighbours", palette.name=coolBlueHotRed) # areas of low neighbour distance indicate groups of nodes that are similar, 
# areas with large distances refer to boundaries between clusters
colnames(som.res$code) # plots normalised component pl.
plot(som.res, type = "property", property = som.res$codes[,"PROX"], main=c("Proximity"), palette.name=coolBlueHotRed) #
plot(som.res, type = "property", property = som.res$codes[,"DENS"], main=c("Density"), palette.name=coolBlueHotRed) #

# cluster component planes through k-means and
# determine number of clusters by means of minimizing the 
# within cluster sum of squares
som.cluster <- som.res$codes 
wss <- (nrow(som.cluster) - 1) * sum(apply(som.cluster, 2, var)) 
for (i in 2:10) {
  set.seed(20082014)
  wss[i] <- sum(kmeans(som.cluster, centers=i)$withinss)
}
plot(wss, type="o", xlab="Number of clusters")

# re-run k-means with 7 clusters
set.seed(20082014)
som.cluster.res <- kmeans(som.cluster, 7)$cluster

# plot the clustered component planes
plot(som.res, type="mapping", bgcol = brewer.pal(7, "Set1")[som.cluster.res], main = "Clusters") 
add.cluster.boundaries(som.res, som.cluster.res)

# plot clusters on a geographical map
cluster_details <- data.frame(id=data$ID, cluster=som.cluster.res[som.res$unit.classif])
data@data <- data.frame(data@data, cluster_details[match(data@data[,"ID"], cluster_details[,"id"]), ])
data$fcluster <- as.factor(data$cluster)
spplot(data, "fcluster", col="transparent", col.regions=brewer.pal(7, "Set1"))

# end
